searchNodes=[{"doc":"PayDayLoan Fast cache now! This project provides a framework for building on-demand caching in Elixir. It provides a synchronous API to a cache that is loaded asynchronously. The cache itself may be backed in any way that you choose, though the default is to use an ETS table backend that has several built-in features for managing the mapping of keys to process ids (e.g., a process registry). You have the option of implementing your own backend using Redis, mnesia, a single process, etc. PDL is designed for low-latency access to cache elements after they are initially loaded and gives you a framework to minimize load time by performing batch loads. This works very well with data streaming applications that have multiple workers processing events in parallel and are sharing cache state across workers. Think of PDL as a cache &quot;frontend&quot;. In a typical application, we may want to load data from a database and cache it for fast lookup later. PDL provides a &quot;frontend&quot; so that MyCache.get(some_id) will automatically make sure that the data corresponding to some_id is loaded into the cache and will return the value once it is available (or time out if the load takes too long). It batches the loading of data so that you can take advantage of, e.g., database queries that fetch multiple records in one call. The actual storage of the data is done by a cache &quot;backend&quot;. PDL provides a default backend via PayDayLoan.EtsBackend that is quite flexible. You can, however, implement your own backend using the PayDayLoan.Backend behaviour. This is useful for using an external service (e.g., Redis) as a cache backend. See the examples below. NOTE _pid functions (e.g., PayDayLoan.get_pid/2 ) are deprecated and have been removed. These functions can be replaced with their non- _pid equivalents. get_pid is replaced with get , peek_pid is replaced with peek , and with_pid is replaced with with_value . 0.3.0 was the last release that included the _pid functions. Key ideas Presents a synchronous API for asynchronous cache loading The cache consists of key-value pairs Provides a default backend for storing values in an ETS table but allows arbitrary backend implementations Tries very hard not to use process messaging in the main lookup API because that can be a bottleneck. Uses ETS tables for state management. Encourages bulk queries for cache loading. Provides hooks for instrumentation Example usage: Default backend # cache wrapper module - this wraps the PDL functions so that they # make sense within the context of your application defmodule MyCache do # defines MyCache.pay_day_loan/0 (and alias pdl/0), # which is set up with defaults and the supplied callback module use PayDayLoan , callback_module : MyCacheLoader # optionally pass in other arguments to override defaults, e.g., # use PayDayLoan, callback_module: MyCacheLoader, batch_size: 100 # also defines pass-through functions for the PayDayLoan module - # e.g., `MyCache.get(key)` is a pass-through to # `MyCache.get(MyCache.pdl(), key)` end # cache loader callback module - this will, for example, execute database # queries and turn the results into cache elements (e.g., Agent or # GenServer processes) defmodule MyCacheLoader do @behaviour PayDayLoan.Loader def key_exists? ( key ) do # should return true if the key exists - # e.g., if &quot;SELECT count(1) FROM some_table WHERE id = \#{key}&quot; returns &gt; 0 end def bulk_load ( keys ) do # code to look up records for keys in database (or whatever) # should return a list of tuples of the format # [{key, load_datum}] end def new ( key , load_datum ) do # note these are three separate examples - your callback will not do # all three # if we are using processes: Agent . start_link ( fn -&gt; load_datum end ) # if we want to store a callback: { :ok , fn -&gt; { :ok , load_datum } end } # if we want to store the bare value { :ok , load_datum } end def refresh ( existing_value , key , load_datum ) do # note these are three separate examples - your callback will not do # all three # if we are using proccesses, the existing_value is the pid of the # already-started process pid = existing_value Agent . update ( pid , fn ( _cached_datum ) -&gt; load_datum end ) # we need to return the pid back { :ok , pid } # or we could stop the existing pid and replace it with a new one Agent . stop ( pid ) Agent . start_link ( fn -&gt; load_datum end ) # or if we stored a callback { :ok , cached_datum } = existing_value . ( ) Logger . info ( &quot;Replacing \#{ inspect cached_datum } with \#{ inspect load_datum } &quot; ) { :ok , fn -&gt; { :ok , load_datum } end } # or to store the new datum as a bare value { :ok , load_datum } end end # Add PDL to your existing supervision tree so that everything initializes properly defmodule MyOTPApp do use Application # existing Application.start callback def start ( _type , _args ) do my_supervisor_children = [ # ... existing children specs PayDayLoan . supervisor_specification ( MyCache . pdl ) ] # for example Supervisor . start_link ( my_supervisor_children , supervisor_opts ) end end # synchronous API - behind the scenes will add the key (1) to the # load state table and the asynchronous loader will include that # in its next load cycle - this call does not return until either # the cache is loaded (via new above) or the request times out { :ok , value } = MyCache . get ( 1 ) Example usage: Process backend (e.g., Redis connection) # cache wrapper module - this wraps the PDL functions so that they # make sense within the context of your application defmodule MyCache do # same as above but we specify a `backend` module and disable the # cache monitor, we also specify a `backend_payload` so that we can # specify a unique identifier for the backend process use ( PayDayLoan , callback_module : MyCacheLoader , backend : MyCacheBackend , backend_payload : :my_cache , cache_monitor : false # we won&#39;t be storing pids ) end # same ideas as above but the new/refresh callbacks are different defmodule MyCacheLoader do @behaviour PayDayLoan.Loader def key_exists? ( key ) do # should return true if the key exists - # e.g., if &quot;SELECT count(1) FROM some_table WHERE id = \#{key}&quot; returns &gt; 0 end def bulk_load ( keys ) do # code to look up records for keys in database (or whatever) # should return a list of tuples of the format # [{key, load_datum}] end def new ( key , load_datum ) do # we could modify the data here, but we are just going to store it raw { :ok , load_datum } end def refresh ( _existing_value , key , load_datum ) do # we could merge the existing value and the load_datum or we could modify # before we store, but we&#39;re just going to replace { :ok , load_datum } end end # backend behaviour implementation defmodule MyCacheBackend do @behaviour PayDayLoan.Backend # this shows an example of how we might use a single process backend, using # Redis is very similar - the process would be Redis connection and the # various callbacks would use Redis commands def start_link ( name ) , do : Agent . start_link ( fn -&gt; %{ } end , name : __name ) # nothing to do for setup def setup ( _pdl ) , do : :ok # this would be a little more involved with redis - you could use the KEYS # command and then MGET but with a large cache, that approach is not # advised. SCAN can be used with larger caches. def reduce ( pdl , acc0 , reducer ) do Agent . get ( pdl . backend_payload , fn ( m ) -&gt; Enum . reduce ( m , acc0 , reducer ) end ) end # with redis this could be a call to DBSIZE def size ( pdl ) , do : Agent . get ( pdl . backend_payload , &amp; map_size / 1 ) # with redis this could be a call to the KEYS command def keys ( pdl ) , do : Agent . get ( pdl . backend_payload , &amp; Map . keys / 1 ) # see comments on the reduce command def values ( pdl ) , do : Agent . get ( pdl . backend_payload , &amp; Map . values / 1 ) # this should be a simple GET command in redis def get ( pdl , key ) do case Agent . get ( pdl . backend_payload , fn ( m ) -&gt; Map . get ( m , key ) end ) do nil -&gt; { :error , :not_found } v -&gt; { :ok , v } end end # with redis you could use SET here def put ( pdl , key , val ) do Agent . update ( pdl . backend_payload , fn ( m ) -&gt; Map . put ( m , key , &quot;V \#{ val } &quot; ) end ) end # corresponds to redis DEL def delete ( pdl , key ) do Agent . update ( pdl . backend_payload , fn ( m ) -&gt; Map . delete ( m , key ) end ) end end # Add PDL to your existing supervision tree so that everything initializes properly defmodule MyOTPApp do use Application # existing Application.start callback def start ( _type , _args ) do my_supervisor_children = [ # start the backend with the payload as its name worker ( MyCacheBackend , [ MyCache . pdl ( ) . backend_payload ] ) , # ... existing children specs PayDayLoan . supervisor_specification ( MyCache . pdl ) ] # for example Supervisor . start_link ( my_supervisor_children , supervisor_opts ) end end # synchronous API - behind the scenes will add the key (1) to the # load state table and the asynchronous loader will include that # in its next load cycle - this call does not return until either # the cache is loaded (via new above) or the request times out { :ok , value } = MyCache . get ( 1 ) Logging &amp; Instrumentation The use macro accepts an event_loggers option, which should be a list of functions that take two arguments. When certain events occur, each of these functions will be called with an event atom and the key requested. The events are :timed_out - Timed out while loading cache. :disappeared - Key was marked as :loaded but the backend did not return a value :failed - The loader failed to load a value for the key :cache_miss - A requested value was not already cached :no_key - The loaded says this key does not exist Example usage: defmodule CacheEventLogger do require Logger def log ( event , key ) do Logger . debug ( &quot;Requesting key \#{ inspect key } caused event \#{ inspect event } &quot; ) end end defmodule CacheEventStats do def log ( event , key ) do # update a statsd counter, etc. end end defmodule MyCache do use PayDayLoan , event_loggers : [ &amp; CacheEventLogge . log / 2 , &amp; CacheEventStats . log / 2 ] end The PayDayLoan.load_state_stats/1 function returns the count of keys in each load state and is also useful for instrumentation. Development &amp; Contributing The usual Elixir and github contribution workflows apply. Pull requests are welcome! mix deps.get mix compile mix test License See LICENSE.txt","ref":"PayDayLoan.html","title":"PayDayLoan","type":"module"},{"doc":"Mixin support for generating a cache. Example: defmodule MyCache do use PayDayLoan , callback_module : MyCacheLoader end The above would define MyCache.pay_day_loan/0 , which returns a PDL struct that is configured for this cache and has callback module MyCacheLoader . Other keys of the %PayDayLoan{} struct can be passed in as options to override the defaults. Also defines pass-through convenience functions for every function in PayDayLoan .","ref":"PayDayLoan.html#__using__/1","title":"PayDayLoan.__using__/1","type":"macro"},{"doc":"Manually add a single key/pid to the cache. Fails if the key is already in cache with a different pid.","ref":"PayDayLoan.html#cache/3","title":"PayDayLoan.cache/3","type":"function"},{"doc":"Synchronously get the value for a key, attempting to load it if it is not alraedy loaded.","ref":"PayDayLoan.html#get/2","title":"PayDayLoan.get/2","type":"function"},{"doc":"Returns a list of all keys in the given cache","ref":"PayDayLoan.html#keys/1","title":"PayDayLoan.keys/1","type":"function"},{"doc":"Returns a map of load states and the number of keys in each state Useful for instrumentation","ref":"PayDayLoan.html#load_state_stats/1","title":"PayDayLoan.load_state_stats/1","type":"function"},{"doc":"Check for a cached value, but do not request a load","ref":"PayDayLoan.html#peek/2","title":"PayDayLoan.peek/2","type":"function"},{"doc":"Check load state, but do not request a load","ref":"PayDayLoan.html#peek_load_state/2","title":"PayDayLoan.peek_load_state/2","type":"function"},{"doc":"Returns a list of all pids in the given cache","ref":"PayDayLoan.html#pids/1","title":"PayDayLoan.pids/1","type":"function"},{"doc":"Check load state, request load if not loaded or loading Does not ping the load worker. A load will not happen until the next ping. Use request_load/2 to request load and trigger a load ping.","ref":"PayDayLoan.html#query_load_state/2","title":"PayDayLoan.query_load_state/2","type":"function"},{"doc":"Perform Enum.reduce/3 over all {key, pid} pairs in the given cache","ref":"PayDayLoan.html#reduce/3","title":"PayDayLoan.reduce/3","type":"function"},{"doc":"Request a load of one or more keys. Load is asynchronous - this function returns immediately","ref":"PayDayLoan.html#request_load/2","title":"PayDayLoan.request_load/2","type":"function"},{"doc":"Returns the number of keys in the given cache","ref":"PayDayLoan.html#size/1","title":"PayDayLoan.size/1","type":"function"},{"doc":"Returns a supervisor specification for the given pdl","ref":"PayDayLoan.html#supervisor_specification/1","title":"PayDayLoan.supervisor_specification/1","type":"function"},{"doc":"Remove a key without killing the underlying process. If you want to remove an element from cache, just kill the underlying process.","ref":"PayDayLoan.html#uncache_key/2","title":"PayDayLoan.uncache_key/2","type":"function"},{"doc":"Return all of the values stored in the backend","ref":"PayDayLoan.html#values/1","title":"PayDayLoan.values/1","type":"function"},{"doc":"Execute a callback with a value if it is found. If no value is found, not_found_callback is executed. By default, the not_found_callback is a function that returns {:error, :not_found} .","ref":"PayDayLoan.html#with_value/4","title":"PayDayLoan.with_value/4","type":"function"},{"doc":"Error values that may be returned from get/2 :not_found - The key is not found as per the key_exists? loader callback :timed_out - Timed out waiting for the value to load. :failed - Either the new or refresh callback failed or returned :ignore . Note - failure state clears when the get function returns. Further calls to get will retry a load.","ref":"PayDayLoan.html#t:error/0","title":"PayDayLoan.error/0","type":"type"},{"doc":"An event that can happen on cache request. :timed_out - Timed out while loading cache. :disappeared - Key was marked as :loaded but the backend did not return a value :failed - The loader failed to load a value for the key :cache_miss - A requested value was not already cached :no_key - The loaded says this key does not exist","ref":"PayDayLoan.html#t:event/0","title":"PayDayLoan.event/0","type":"type"},{"doc":"A function that takes an event and a key and performs some logging action. The return value is ignored","ref":"PayDayLoan.html#t:event_logger/0","title":"PayDayLoan.event_logger/0","type":"type"},{"doc":"A key in the cache. This could be any Erlang/Elixir term . In practice, for example, it may be an integer representing the primary key in a database table.","ref":"PayDayLoan.html#t:key/0","title":"PayDayLoan.key/0","type":"type"},{"doc":"Datum returned by the load callback corresponding to a single key. For example, this could be a tuple of database column values or a struct encapsulating such values. Your new and refresh loader callbacks should know how to ingest these values to generate new cache entry processes.","ref":"PayDayLoan.html#t:load_datum/0","title":"PayDayLoan.load_datum/0","type":"type"},{"doc":"Struct encapsulating a PDL cache. backend - Implementation of the Backend behaviour - defaults to PayDayLoan.EtsBackend. backend_payload - Arbitrary payload for the backend - defaults to the ETS table id for the ETS backend. load_state_manager - ETS table id for load state table. cache_monitor - Registration name for the monitor process, or false if no monitor should be started. key_cache - ETS table id for key cache table. load_worker - Registration name for the load worker GenServer. callback_module - Module implementing the PayDayLoan.Loader behaviour. batch_size - Maximum number of keys to load at once. Default 1000 load_num_tries - Maximum number of times to wait for cache load. Default 10 load_wait_msec - Amount of time to wait between checking load state. Default 500 supervisor_name - Registration name for the supervisor.","ref":"PayDayLoan.html#t:t/0","title":"PayDayLoan.t/0","type":"type"},{"doc":"Defines the PDL backend behaviour. The backend is responsible for allowing us to access the cached value for a given key. PayDayLoan's EtsBackend module is used by default, and is built with a process cache in mind. You could use the Backend behaviour to implement, for example, a Redis or mnesia backend.","ref":"PayDayLoan.Backend.html","title":"PayDayLoan.Backend","type":"behaviour"},{"doc":"Delete a key from the cache Must return :ok","ref":"PayDayLoan.Backend.html#c:delete/2","title":"PayDayLoan.Backend.delete/2","type":"callback"},{"doc":"Retrieve a value from the cache Should return {:error, :not_found} if the value is not found and {:ok, value} otherwise.","ref":"PayDayLoan.Backend.html#c:get/2","title":"PayDayLoan.Backend.get/2","type":"callback"},{"doc":"Should return a list of keys in the cache","ref":"PayDayLoan.Backend.html#c:keys/1","title":"PayDayLoan.Backend.keys/1","type":"callback"},{"doc":"Insert a value into the cache Must return :ok","ref":"PayDayLoan.Backend.html#c:put/3","title":"PayDayLoan.Backend.put/3","type":"callback"},{"doc":"Should execute a reduce operation over the key/value pairs in the cache. The reducer function should take arguments in the form ({key, value}, accumulator) and return the new accumulator value - i.e., it should operate as Enum.reduce/3 would when given a map.","ref":"PayDayLoan.Backend.html#c:reduce/3","title":"PayDayLoan.Backend.reduce/3","type":"callback"},{"doc":"Called during supervisor initialization - use this callback to initialize your backend if needed. Must return :ok .","ref":"PayDayLoan.Backend.html#c:setup/1","title":"PayDayLoan.Backend.setup/1","type":"callback"},{"doc":"Should return the number of keys in the cache","ref":"PayDayLoan.Backend.html#c:size/1","title":"PayDayLoan.Backend.size/1","type":"callback"},{"doc":"Should return a list of values in the cache","ref":"PayDayLoan.Backend.html#c:values/1","title":"PayDayLoan.Backend.values/1","type":"callback"},{"doc":"ETS-based backend capable of handling raw values, pids, or callbacks. This is the default backend used by PayDayLoan and is designed for storing process ids. However, it can be used with raw values or callback functions. With pids, special care is taken to keep the cache state consistent with the &quot;alive&quot; state of the processes. If a process is found to be dead, the key is removed from cache. The PayDayLoan.ProcessMonitor process monitors pids, and we check for alive-ness whenever we resolve a value. If a callback is stored, then the callback is executed whenever we attempt to resolve a value - e.g., on get or reduce or values calls. The callback must return a tuple with {:ok, value} on success or {:error, :not_found} on failure. The functions in this module are documented only to aid in understanding how the default backend works. They should not be called directly - only through the PDL API.","ref":"PayDayLoan.EtsBackend.html","title":"PayDayLoan.EtsBackend","type":"module"},{"doc":"Remove a key from cache","ref":"PayDayLoan.EtsBackend.html#delete/2","title":"PayDayLoan.EtsBackend.delete/2","type":"function"},{"doc":"Remove a value from cache","ref":"PayDayLoan.EtsBackend.html#delete_value/2","title":"PayDayLoan.EtsBackend.delete_value/2","type":"function"},{"doc":"Get the value corresponding to the given key If the value is a process that is not alive, deletes the entry and returns {:error, :not_found} .","ref":"PayDayLoan.EtsBackend.html#get/2","title":"PayDayLoan.EtsBackend.get/2","type":"function"},{"doc":"Returns a list of all cached keys","ref":"PayDayLoan.EtsBackend.html#keys/1","title":"PayDayLoan.EtsBackend.keys/1","type":"function"},{"doc":"Add a value to the cache and monitor it if it is a pid.","ref":"PayDayLoan.EtsBackend.html#put/3","title":"PayDayLoan.EtsBackend.put/3","type":"function"},{"doc":"Perform Enum.reduce on the ETS table","ref":"PayDayLoan.EtsBackend.html#reduce/3","title":"PayDayLoan.EtsBackend.reduce/3","type":"function"},{"doc":"Setup callback, creates the underlying ETS table","ref":"PayDayLoan.EtsBackend.html#setup/1","title":"PayDayLoan.EtsBackend.setup/1","type":"function"},{"doc":"Returns the number of cached keys","ref":"PayDayLoan.EtsBackend.html#size/1","title":"PayDayLoan.EtsBackend.size/1","type":"function"},{"doc":"Returns a list of all cached values","ref":"PayDayLoan.EtsBackend.html#values/1","title":"PayDayLoan.EtsBackend.values/1","type":"function"},{"doc":"Keeps track of which keys are known to exist in the cache source You shouldn't need to call these functions manually, but they can be useful for debugging.","ref":"PayDayLoan.KeyCache.html","title":"PayDayLoan.KeyCache","type":"module"},{"doc":"Add a key to the cache","ref":"PayDayLoan.KeyCache.html#add_to_cache/2","title":"PayDayLoan.KeyCache.add_to_cache/2","type":"function"},{"doc":"Returns true if the key exists either in the key cache or source Calls callback_module.key_exists?(key) if the key is not already in cache.","ref":"PayDayLoan.KeyCache.html#exist?/3","title":"PayDayLoan.KeyCache.exist?/3","type":"function"},{"doc":"Returns true if the key is in cache","ref":"PayDayLoan.KeyCache.html#in_cache?/2","title":"PayDayLoan.KeyCache.in_cache?/2","type":"function"},{"doc":"Calls callback_module.key_exists?(key) , adds key to cache if the callback returns true","ref":"PayDayLoan.KeyCache.html#lookup?/3","title":"PayDayLoan.KeyCache.lookup?/3","type":"function"},{"doc":"Remove a key from the cache","ref":"PayDayLoan.KeyCache.html#remove/2","title":"PayDayLoan.KeyCache.remove/2","type":"function"},{"doc":"Keeps track of which keys are loaded, requested, and loading Acts as a state tracker and a queue for the loader. You shouldn't need to call any of these functions manually but they can be useful for debugging.","ref":"PayDayLoan.LoadState.html","title":"PayDayLoan.LoadState","type":"module"},{"doc":"Returns all elements of the table","ref":"PayDayLoan.LoadState.html#all/1","title":"PayDayLoan.LoadState.all/1","type":"function"},{"doc":"Returns true if any keys are in the :requested or :reload states","ref":"PayDayLoan.LoadState.html#any_requested?/1","title":"PayDayLoan.LoadState.any_requested?/1","type":"function"},{"doc":"Set state to :failed","ref":"PayDayLoan.LoadState.html#failed/2","title":"PayDayLoan.LoadState.failed/2","type":"function"},{"doc":"Set state to :loaded","ref":"PayDayLoan.LoadState.html#loaded/2","title":"PayDayLoan.LoadState.loaded/2","type":"function"},{"doc":"Set state to :loading","ref":"PayDayLoan.LoadState.html#loading/2","title":"PayDayLoan.LoadState.loading/2","type":"function"},{"doc":"Return load state without modifying; return nil if key is not found","ref":"PayDayLoan.LoadState.html#peek/2","title":"PayDayLoan.LoadState.peek/2","type":"function"},{"doc":"Set the load state to :requested if not loaded or loading, return the load state","ref":"PayDayLoan.LoadState.html#query/2","title":"PayDayLoan.LoadState.query/2","type":"function"},{"doc":"Set the state to :reload","ref":"PayDayLoan.LoadState.html#reload/2","title":"PayDayLoan.LoadState.reload/2","type":"function"},{"doc":"Return the list of keys in the :reload state, limited to limit elements","ref":"PayDayLoan.LoadState.html#reload_keys/2","title":"PayDayLoan.LoadState.reload_keys/2","type":"function"},{"doc":"Set state to :reload_loading","ref":"PayDayLoan.LoadState.html#reload_loading/2","title":"PayDayLoan.LoadState.reload_loading/2","type":"function"},{"doc":"Set state to :requested","ref":"PayDayLoan.LoadState.html#request/2","title":"PayDayLoan.LoadState.request/2","type":"function"},{"doc":"Set the state to :reload if the key is loaded, set it to :request if it is not","ref":"PayDayLoan.LoadState.html#request_or_reload/2","title":"PayDayLoan.LoadState.request_or_reload/2","type":"function"},{"doc":"Return the list of requested keys, limited to limit elements","ref":"PayDayLoan.LoadState.html#requested_keys/2","title":"PayDayLoan.LoadState.requested_keys/2","type":"function"},{"doc":"Remove a key from the load state table","ref":"PayDayLoan.LoadState.html#unload/2","title":"PayDayLoan.LoadState.unload/2","type":"function"},{"doc":"Load states that a key can have. :requested - A load has been requested. The load worker should pick this up and set the state to :loading . :reload - A value is available and a reload has been requested. :loading - The load worker is in the process of loading this key. :reload_loading - The load worker is in the process of loading this key, but it is already loaded and will not block. :loaded - The key is loaded in cache. :failed - The key attempted a load or refresh and failed.","ref":"PayDayLoan.LoadState.html#t:t/0","title":"PayDayLoan.LoadState.t/0","type":"type"},{"doc":"Process to load requested keys into PDL cache. Whenever this process receives a ping, it attempts to load a batch of keys. Requested keys are ones returned by a call to LoadState.requested_keys . Pings should happen automatically via the PDL API. To force a ping manually, call GenServer.cast(pdl.load_worker, :ping) .","ref":"PayDayLoan.LoadWorker.html","title":"PayDayLoan.LoadWorker","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"PayDayLoan.LoadWorker.html#child_spec/1","title":"PayDayLoan.LoadWorker.child_spec/1","type":"function"},{"doc":"Start in a supervision tree","ref":"PayDayLoan.LoadWorker.html#start_link/1","title":"PayDayLoan.LoadWorker.start_link/1","type":"function"},{"doc":"","ref":"PayDayLoan.LoadWorker.html#t:state/0","title":"PayDayLoan.LoadWorker.state/0","type":"type"},{"doc":"Defines the PDL loader behaviour. The loader is responsible for fetching a batch of data ( bulk_load/1 ) from the cache's source (e.g., a database). For each element that is fetched, we check if there is an existing element in the cache backend. If no existing element is found for a key, we call new/2 with the corresponding key and value. This gives the loader an opportunity to alter what is stored in the backend from what is returned by the loader. For example, we may be using a process backend and all we need to store is the pid. If an existing value is found for a key, we call refresh/3 with the existing backend value, the key, and the load datum. For example, with a process backend this may involve making a GenServer or Agent update call before returning the pid to the backend. The key_exists?/1 function is a hook for the key cache.","ref":"PayDayLoan.Loader.html","title":"PayDayLoan.Loader","type":"behaviour"},{"doc":"Fetch values to be loaded into cache. For example, perform a database lookup and return the data normalized in some form that your cache elements will understand. new or refresh will be called for each returned element as appropriate. Note that there is no requirement that each key passed in be represented in the output.","ref":"PayDayLoan.Loader.html#c:bulk_load/1","title":"PayDayLoan.Loader.bulk_load/1","type":"callback"},{"doc":"Should return true if a key exists in the cache's source. This should be a fast-executing call - for example, with a database source - SELECT count(1) FROM cache_source WHERE id = \#{key} . You can stub this method to always return true if you want to effectively skip key caching.","ref":"PayDayLoan.Loader.html#c:key_exists?/1","title":"PayDayLoan.Loader.key_exists?/1","type":"callback"},{"doc":"Create a new cache element before sending it to the backend. For example, this may create a process and the resulting pid is what is actually stored in the backend, or it may simply manipulate the load datum before it is stored. To store the load datum directly in the backend, just return {:ok, load_datum} . To signal an error which will cause the load to fail for this key, return {:error, &lt;error payload&gt;} .","ref":"PayDayLoan.Loader.html#c:new/2","title":"PayDayLoan.Loader.new/2","type":"callback"},{"doc":"Update a cache element before storing it. This is called when a load occurs for a key that already has a value in the cache. You can use this callback to resolve the differences or to update the value before sending it on to the backend. For example, with a process-based cache (e.g. using PayDayLoan.EtsBackend ), the existing_value is the backing processes pid and you may need to call something like Agent.update(existing_value, key, &lt;update function&gt;) and return existing_value unchanged (since the same pid will still be stored on the backend). To pass the load datum directly through to the backend, return {:ok, load_datum} . To signal an error, return {:error, &lt;error payload&gt;} . To ignore this datum, return :ignore . In both the :ignore and :error cases, the key will not be reloaded until it is requested again.","ref":"PayDayLoan.Loader.html#c:refresh/3","title":"PayDayLoan.Loader.refresh/3","type":"callback"},{"doc":"Monitor for process registry caches. This is built to work with the supplied PayDayLoan.EtsBackend backend, for which it monitors pids and removes them from cache if the pid dies. This can be disabled by setting cache_monitor: false in the PDL setup.","ref":"PayDayLoan.ProcessMonitor.html","title":"PayDayLoan.ProcessMonitor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"PayDayLoan.ProcessMonitor.html#child_spec/1","title":"PayDayLoan.ProcessMonitor.child_spec/1","type":"function"},{"doc":"Callback implementation for GenServer.init/1 .","ref":"PayDayLoan.ProcessMonitor.html#init/1","title":"PayDayLoan.ProcessMonitor.init/1","type":"function"},{"doc":"Supervisor for PDL processes. You can either start this manually or use PayDayLoan.supervisor_specification/1 to return a supervisor spec that can be passed into another supervisor's start_link call (recommended).","ref":"PayDayLoan.Supervisor.html","title":"PayDayLoan.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"PayDayLoan.Supervisor.html#child_spec/1","title":"PayDayLoan.Supervisor.child_spec/1","type":"function"},{"doc":"Callback implementation for Supervisor.init/1 .","ref":"PayDayLoan.Supervisor.html#init/1","title":"PayDayLoan.Supervisor.init/1","type":"function"},{"doc":"Start in a supervision tree","ref":"PayDayLoan.Supervisor.html#start_link/1","title":"PayDayLoan.Supervisor.start_link/1","type":"function"}]